/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL07 for C using RDMSQL */
/* This example shows how to use parameters with SQL commands.  A database
 * is created and then data is inserted with positioned parameters and then
 * with named parameters.  Finally the data is read out and displayed.
 *
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *      rsqlPrepare
 *      rsqlBindParam
 *      rsqlBindNamedParam
 *      rsqlExecute
 *      rsqlCloseStmt
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity, this example does not check all return codes, but good
 * programming practices would dictate that they are checked after each
 * RDMSQL call.
 */
static void drop_database(
    HCONN       hDbc,
    const char *name)
{
    HSTMT hStmt;
    char  commandText[80];

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    sprintf(commandText, "DROP DATABASE %s", name);
    (void) rsqlExecDirect(hStmt, commandText);
    (void) rsqlExecDirect(hStmt, "COMMIT");

    (void) rsqlFreeStmt(hStmt);
}

static void create_this_database(
    HCONN       hDbc,
    const char *name,
    const char *cmds[],
    uint32_t    cmd_length)
{
    RSQL_ERRCODE rsqlCode;
    HSTMT        hStmt;
    char         commandText[80];
    uint16_t     ii;

    (void) rsqlAllocStmt(hDbc, &hStmt);

    sprintf(commandText, "CREATE DATABASE %s", name);
    rsqlCode = rsqlExecDirect(hStmt, commandText);

    for (ii = 0; ii < cmd_length; ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the %s database\n", name);

    (void) rsqlExecDirect(hStmt, "COMMIT");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, name);

    (void) rsqlFreeStmt(hStmt);
}

static void create_databases(
    HCONN hDbc)
{
    const char *cmds[] = {
        "CREATE TABLE Table1 (id INTEGER PRIMARY KEY, str CHAR(30))"
    };

    /* Drop any existing copies of the databases. */
    drop_database(hDbc, "SQL07_db");

    /* Attempt to create the databases */
    create_this_database(hDbc, "SQL07_db", cmds, 1);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    int32_t           param_id;
    char              param_str[31];
    int16_t           ii;

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);

    create_databases(hDbc);

    /* Specify which databases to use */
    rsqlCode = rsqlOpenDB(hDbc, "SQL07_db", "x");

    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Insert three rows into the table using positioned parameters */
    rsqlCode = rsqlPrepare(hStmt, "INSERT INTO Table1 VALUES ?, ?");
    if (rsqlCode == errSUCCESS)
    {
        (void) rsqlBindParam(hStmt, 1, tINTEGER, &param_id, NULL);
        (void) rsqlBindParam(hStmt, 2, tCHAR, &param_str, NULL);
    }

    for (ii = 1; ii < 4; ii++)
    {
        param_id = ii;
        sprintf(param_str, "positioned %hd", ii);
        rsqlCode = rsqlExecute(hStmt);
    }

    /* Insert three rows into the table using named parameters */
    (void) rsqlCloseStmt(hStmt);
    rsqlCode = rsqlPrepare(hStmt, "INSERT INTO Table1 VALUES :idparm, :strparm");
    if (rsqlCode == errSUCCESS)
    {
        (void) rsqlBindNamedParam(hStmt, "idparm", tINTEGER, &param_id, NULL, NULL);
        (void) rsqlBindNamedParam(hStmt, "strparm", tCHAR, param_str, NULL, NULL);
    }

    for (ii = 4; ii < 7; ii++)
    {
        param_id = ii;
        sprintf(param_str, "positioned %hd", ii);
        rsqlCode = rsqlExecute(hStmt);
    }

    /* Scan through the data, generating the result */
    (void) rsqlCloseStmt(hStmt);
    (void) rsqlExecDirect(hStmt, "SELECT id, str FROM Table1");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%d -- %s\n", result[0].vt.lv, result[1].vt.cv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc, "SQL07_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

